<?php
/**
 * Loulou66
 * LpsCategoryProducts module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsCategoryProducts extends Module implements WidgetInterface
{
    public $templateFile;

    public function __construct()
    {
        $this->name = 'lpscategoryproducts';
        $this->author = 'Loulou66';
        $this->version = '8.0.1';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS Products in the same category Carousel');
        $this->description = $this->l('Add a block displaying the products in the same category as a CAROUSEL');
        $this->templateFile = 'module:lpscategoryproducts/views/templates/hook/lpscategoryproducts.tpl';
        $this->module_key = 'a5d8413454fec5a5b5c2434dc08d4b7e';
    }

    public function install()
    {
        $this->_clearCache('*');
        return parent::install()
            && Configuration::updateValue('LPS_CAT_PROD_ENABLE', 1)
            && Configuration::updateValue('LPS_CAT_PROD_PROD_DISPLAY', 20)
            && Configuration::updateValue('LPS_CAT_PROD_PROD_PER_ROW', 4)
            && Configuration::updateValue('LPS_CAT_PROD_LOOP', 1)
            && Configuration::updateValue('LPS_CAT_PROD_AUTOPLAY', 1)
            && Configuration::updateValue('LPS_CAT_PROD_RANDOM', 0)
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayFooterProduct')
            && ProductSale::fillProductSales();
    }

    public function uninstall()
    {
        $this->_clearCache('*');
        return parent::uninstall()
            && Configuration::deleteByName('LPS_CAT_PROD_ENABLE')
            && Configuration::deleteByName('LPS_CAT_PROD_PROD_DISPLAY')
            && Configuration::deleteByName('LPS_CAT_PROD_PROD_PER_ROW')
            && Configuration::deleteByName('LPS_CAT_PROD_LOOP')
            && Configuration::deleteByName('LPS_CAT_PROD_AUTOPLAY')
            && Configuration::deleteByName('LPS_CAT_PROD_RANDOM');
    }

    public function isUsingNewTranslationSystem()
    {
        return false;
    }

    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        return parent::_clearCache($this->templateFile);
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules'
            && Tools::getValue('configure') == $this->name
        ) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }

    public function getContent()
    {
        $html = '';
        if (Tools::isSubmit('submitlpscategoryproducts')) {
            if (!Validate::isInt((int) Tools::getValue('LPS_CAT_PROD_PROD_DISPLAY'))
                || (int) Tools::getValue('LPS_CAT_PROD_PROD_DISPLAY') <= 0
            ) {
                $html .= $this->displayError(
                    $this->l('The « Products to display » field is invalid. Please enter a positive number.')
                );
            } else {
                Configuration::updateValue('LPS_CAT_PROD_ENABLE', (int) Tools::getValue('LPS_CAT_PROD_ENABLE'));
                Configuration::updateValue(
                    'LPS_CAT_PROD_PROD_DISPLAY',
                    (int) Tools::getValue('LPS_CAT_PROD_PROD_DISPLAY')
                );
                Configuration::updateValue(
                    'LPS_CAT_PROD_PROD_PER_ROW',
                    (int) Tools::getValue('LPS_CAT_PROD_PROD_PER_ROW')
                );
                Configuration::updateValue('LPS_CAT_PROD_LOOP', (int) Tools::getValue('LPS_CAT_PROD_LOOP'));
                Configuration::updateValue('LPS_CAT_PROD_AUTOPLAY', (int) Tools::getValue('LPS_CAT_PROD_AUTOPLAY'));
                Configuration::updateValue('LPS_CAT_PROD_RANDOM', (int) Tools::getValue('LPS_CAT_PROD_RANDOM'));
                $this->_clearCache('*');
                $html .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $html . $this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable carousel'),
                        'name' => 'LPS_CAT_PROD_ENABLE',
                        'is_bool' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'LPS_CAT_PROD_PROD_DISPLAY',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'form_group_class' => 'LPS_CAT_PROD_ENABLE',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdPerRow(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                        'form_group_class' => 'LPS_CAT_PROD_ENABLE',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'LPS_CAT_PROD_LOOP',
                        'is_bool' => true,
                        'form_group_class' => 'LPS_CAT_PROD_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'LPS_CAT_PROD_AUTOPLAY',
                        'is_bool' => true,
                        'form_group_class' => 'LPS_CAT_PROD_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'LPS_CAT_PROD_RANDOM',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'form_group_class' => 'LPS_CAT_PROD_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitlpscategoryproducts';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$fields_form]);
    }
    public function getConfigFieldsValues()
    {
        return [
            'LPS_CAT_PROD_ENABLE' => (int) Tools::getValue(
                'LPS_CAT_PROD_ENABLE',
                Configuration::get('LPS_CAT_PROD_ENABLE'),
            ),
            'LPS_CAT_PROD_PROD_DISPLAY' => (int) Tools::getValue(
                'LPS_CAT_PROD_PROD_DISPLAY',
                Configuration::get('LPS_CAT_PROD_PROD_DISPLAY'),
            ),
            'LPS_CAT_PROD_LOOP' => (int) Tools::getValue(
                'LPS_CAT_PROD_LOOP',
                Configuration::get('LPS_CAT_PROD_LOOP'),
            ),
            'LPS_CAT_PROD_AUTOPLAY' => (int) Tools::getValue(
                'LPS_CAT_PROD_AUTOPLAY',
                Configuration::get('LPS_CAT_PROD_AUTOPLAY'),
            ),
            'LPS_CAT_PROD_RANDOM' => (int) Tools::getValue(
                'LPS_CAT_PROD_RANDOM',
                Configuration::get('LPS_CAT_PROD_RANDOM'),
            ),
        ];
    }

    public function displayProdPerRow()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpscpprodperrow.tpl';
        $lpscpprodperrow = $this->context->smarty->createTemplate($tpl);
        $lpscpprodperrow->assign(
            [
                'lpscp_prodperrow' => (int) Tools::getValue(
                    'LPS_CAT_PROD_PROD_PER_ROW',
                    Configuration::get('LPS_CAT_PROD_PROD_PER_ROW'),
                ),
            ]
        );
        return $lpscpprodperrow->fetch();
    }

    public function hookActionFrontControllerSetMedia()
    {
        $controllerFront = $this->context->controller->php_self;
        if ($controllerFront == 'product') {
            if (!$this->active || !Configuration::get('LPS_CAT_PROD_ENABLE')) {
                return false;
            }
            Media::addJsDef(
                [
                    'lpscp_productsPerRow' => Configuration::get('LPS_CAT_PROD_PROD_PER_ROW'),
                    'lpscp_loop' => Configuration::get('LPS_CAT_PROD_LOOP') ? true : false,
                    'lpscp_autoplay' => Configuration::get('LPS_CAT_PROD_AUTOPLAY') ? true : false,
                ]
            );
            $this->context->controller->registerStylesheet(
                'modules-lpscategoryproductsfrontcss',
                'modules/' . $this->name . '/views/css/lpscategoryproductsfront.css'
            );
            $this->context->controller->registerStylesheet(
                'modules-owlcarouselcss',
                'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
            );
            $this->context->controller->registerStylesheet(
                'modules-owlcarouselthemecss',
                'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
            );
            $this->context->controller->registerJavascript(
                'modules-owlcarouseljs',
                'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
            );
            $this->context->controller->registerJavascript(
                'modules-lpscategoryproductsfrontjs',
                'modules/' . $this->name . '/views/js/lpscategoryproducts.js'
            );
        }
    }

    public function renderWidget($hookName, array $configuration)
    {
        $controllerFront = $this->context->controller->php_self;
        if ($controllerFront == 'product') {
            if (!$this->active || !Configuration::get('LPS_CAT_PROD_ENABLE')) {
                return false;
            }
            $params = $this->getInformationFromConfiguration($configuration);
            $lpscategoryproductsrandom = Configuration::get('LPS_CAT_PROD_RANDOM');
            if (!$lpscategoryproductsrandom) {
                if (!$this->isCached($this->templateFile, $params['cache_id'])) {
                    $variables = $this->getWidgetVariables($hookName, $configuration);
                    if (empty($variables)) {
                        return false;
                    }
                    $this->context->smarty->assign($variables);
                }
                return $this->fetch($this->templateFile, $params['cache_id']);
            } else {
                $variables = $this->getWidgetVariables($hookName, $configuration);
                if (empty($variables)) {
                    return false;
                }
                $this->context->smarty->assign($variables);
                return $this->fetch($this->templateFile);
            }
        }
    }

    private function getInformationFromConfiguration($configuration)
    {
        if (empty($configuration['product'])) {
            return false;
        }
        $product = $configuration['product'];
        if ($product instanceof Product) {
            $product = [$product];
            $product['id_product'] = $product['id'];
        }
        $id_product = $product['id_product'];
        $id_category = (isset($configuration['category']->id)
             ? (int) $configuration['category']->id
             : (int) $product['id_category_default']
        );
        if (!empty($id_product) && !empty($id_category)) {
            $cache_id = 'ps_categoryproducts|' . $id_product . '|' . $id_category;
            return [
                'id_product' => $id_product,
                'id_category' => $id_category,
                'cache_id' => $this->getCacheId($cache_id),
            ];
        }
        return false;
    }

    public function getWidgetVariables($hookName, array $configuration)
    {
        $params = $this->getInformationFromConfiguration($configuration);
        if ($params) {
            $lpscategoryproducts = $this->getCategoryProducts($params['id_product'], $params['id_category']);
            if (count($lpscategoryproducts) <= Configuration::get('LPS_CAT_PROD_PROD_PER_ROW')) {
                $this->context->smarty->assign(['lpscp_hidenav' => true]);
            }
            if (!empty($lpscategoryproducts)) {
                return ['lpscategoryproducts' => $lpscategoryproducts];
            }
        }
        return false;
    }

    protected function getCategoryProducts($idProduct, $idCategory)
    {
        $nbProducts = Configuration::get('LPS_CAT_PROD_PROD_DISPLAY') + 1;
        $lpscategoryproducts_cat = new Category((int) $idCategory);
        $searchProvider = new CategoryProductSearchProvider($this->getTranslator(), $lpscategoryproducts_cat);
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $query->setResultsPerPage($nbProducts)->setPage(1);
        if (Configuration::get('LPS_CAT_PROD_RANDOM')) {
            $query->setSortOrder(SortOrder::random());
        } else {
            $query->setSortOrder(new SortOrder('product', 'position', 'asc'));
        }
        $result = $searchProvider->runQuery($context, $query);
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        $lpscategoryproductsres = $result->getProducts();
        foreach ($lpscategoryproductsres as $rawProduct) {
            if ($rawProduct['id_product'] !== $idProduct
                && count($products_for_template) < (int) Configuration::get('LPS_CAT_PROD_PROD_DISPLAY')
            ) {
                $products_for_template[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }
        return $products_for_template;
    }
}
